import { useLazyQuery } from '@apollo/client'
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react'
import { useTranslation } from 'react-i18next'
import { FlatList, Platform, Pressable, SafeAreaView, StyleSheet, View } from 'react-native'

import KeyboardAvoidingViewWrapper from '@holi/core/components/KeyboardAvoidingViewWrapper'
import PermissionsGuard, { PermissionsType } from '@holi/core/components/PermissionsGuard'
import { useSafeAreaInsets } from '@holi/core/helpers/useSafeAreaInsets'
import { useSetScreenOptions } from '@holi/core/navigation/hooks/useScreenOptions'
import { StartChatButton } from '@holi/core/screens/userprofile/components/StartChatButton'
import { UserByNameResponse, userByNameQuery } from '@holi/core/screens/userprofile/queries'
import { User } from '@holi/core/screens/userprofile/types'
import { HoliIcon } from '@holi/icons/src/HoliIcon'
import { SpinningIconContainer } from '@holi/icons/src/SpinningIconContainer'
import { Loader, Search } from '@holi/icons/src/generated'
import HoliBox from '@holi/ui/components/atoms/HoliBox'
import HoliHeading from '@holi/ui/components/atoms/HoliHeading'
import HoliText from '@holi/ui/components/atoms/HoliText'
import HoliAvatar from '@holi/ui/components/molecules/HoliAvatar'
import HoliCheckbox from '@holi/ui/components/molecules/HoliCheckbox'
import HoliTextInput from '@holi/ui/components/molecules/HoliTextInput'
import { isWebTouchDevice, useDebounce } from '@holi/ui/helper'
import { colors, vars } from '@holi/ui/styles/globalVars'
import { useStyleMixins } from '@holi/ui/styles/theme/mixins'

const FIND_USER_BY_NAME_LIMIT = 30
const MIN_USER_NAME_CHARACTERS = 2

const USER_ITEM_HEIGHT = 48
const USER_ITEM_VERTICAL_PADDING = 8

const isWeb = Platform.OS === 'web'

const ChatStartConversationView = () => {
  const { t } = useTranslation()
  const mixins = useStyleMixins()
  const { top: safeAreaInsetTop } = useSafeAreaInsets()

  const [searchUserInputValue, setSearchUserInputValue] = useState('')
  const [users, setUsers] = useState<User[]>([])
  const [selectedUser, setSelectedUser] = useState<User | null>(null)
  const initialInput = useRef(true)

  const setScreenOptions = useSetScreenOptions()

  useEffect(() => {
    setScreenOptions({
      title: t('chat.start.title'),
      headerStyle: {
        backgroundColor: colors.background20,
      },
      navigationCustomOptions: {
        footerShown: false,
      },
    })
  }, [setScreenOptions, t])

  // BE API
  const [fetchUserByName, { loading }] = useLazyQuery<UserByNameResponse>(userByNameQuery)
  const getUsersByName = useCallback(
    async (userNameSubstr: string, offset = 0) => {
      const { data: { userByName: { data: users = [] } = {} } = {} } = await fetchUserByName({
        variables: {
          substring: userNameSubstr,
          offset,
          limit: FIND_USER_BY_NAME_LIMIT,
        },
      })

      setUsers(users)
    },
    [fetchUserByName]
  )
  const getUsersByNameDebounced = useDebounce(getUsersByName, 300)

  // Event handlers
  const onSearchUserInputChange = async (value: string) => {
    setSearchUserInputValue(value)
    // deselect user
    setSelectedUser(null)
    // fetch users only when input is greater equal 2 characters
    if (value.length >= MIN_USER_NAME_CHARACTERS) {
      getUsersByNameDebounced(value)
      if (initialInput.current) {
        initialInput.current = false
      }
    }
  }

  const onUserSelect = (user: User) => {
    setSelectedUser(user.id !== selectedUser?.id ? user : null)
  }

  const trailingComponent = useMemo(() => {
    const icon = loading ? Loader : Search
    return (
      <SpinningIconContainer spinning={loading}>
        <HoliIcon icon={icon} size={22} />
      </SpinningIconContainer>
    )
  }, [loading])

  return (
    <PermissionsGuard permissionType={PermissionsType.AUTH}>
      <KeyboardAvoidingViewWrapper
        renderProp={() => (
          <SafeAreaView style={styles.wrapper}>
            <View style={[styles.inner, { paddingTop: Platform.OS === 'android' ? safeAreaInsetTop : 0 }]}>
              <HoliTextInput
                value={searchUserInputValue}
                placeholder={t('chat.start.input.label')}
                aria-label={t('chat.start.input.label')}
                trailingComponent={trailingComponent}
                onChange={onSearchUserInputChange}
                validationErrorText={
                  !initialInput.current && searchUserInputValue.length < MIN_USER_NAME_CHARACTERS
                    ? t('chat.start.input.error', { count: MIN_USER_NAME_CHARACTERS })
                    : ''
                }
                style={mixins.container}
              />

              {users.length ? (
                <FlatList
                  data={users}
                  renderItem={({ item: user }) => {
                    return (
                      <Pressable style={[styles.flexRow, styles.userItem]} onPress={() => onUserSelect(user)}>
                        <View style={[styles.flexRow, styles.userItemContent]}>
                          <HoliBox padding={[0, 16, 0, 0]}>
                            <HoliAvatar
                              imageSrc={user.avatar}
                              imageBlurhash={user.avatarBlurhash}
                              label={user.avatarLabel}
                              initials={user.avatarLabel}
                              defaultColor={user.avatarDefaultColor}
                              size="medium"
                            />
                          </HoliBox>

                          <HoliText>{user.fullName}</HoliText>
                        </View>

                        <HoliBox padding={[0, 0, 0, 16]}>
                          <HoliCheckbox
                            shape="round"
                            checked={user.id === selectedUser?.id}
                            autofocus={false}
                            aria-label={t('chat.start.userItem.checkbox.label', { user: user.fullName })}
                            onSelect={() => onUserSelect(user)}
                          />
                        </HoliBox>
                      </Pressable>
                    )
                  }}
                  keyExtractor={(item) => item.id}
                  ListFooterComponent={() => <View style={{ height: 98 }} />}
                  getItemLayout={(_, index) => ({
                    length: USER_ITEM_HEIGHT,
                    offset: (USER_ITEM_HEIGHT + USER_ITEM_VERTICAL_PADDING * 2) * index,
                    index,
                  })}
                  style={[mixins.container, styles.list]}
                  keyboardShouldPersistTaps="handled"
                />
              ) : (
                <View style={[mixins.container, styles.listDesctiption]}>
                  <HoliHeading level="3" style={styles.listDesctiptionText}>
                    {t('chat.start.description')}
                  </HoliHeading>
                </View>
              )}

              <View style={[mixins.container, styles.overlayButtonWrapper]}>
                <StartChatButton user={selectedUser} label={t('chat.start.button.label')} doReplaceRoute={true} />
              </View>
            </View>
          </SafeAreaView>
        )}
      />
    </PermissionsGuard>
  )
}

export default ChatStartConversationView

const styles = StyleSheet.create({
  wrapper: {
    backgroundColor: colors.background20,
    flex: 1,
    ...Platform.select({
      web: {
        minHeight: '100%',
      },
    }),
  },
  inner: {
    flex: 1,
    // 16 is space between the header and the input
    // we do not use `paddingTop` for this spacing, because it will be overriden with `safeAreaInsetTop` for Android
    marginTop: +vars.topBarHeight + 16,
  },
  list: {
    flex: 1,
    paddingTop: 24,
  },
  listDesctiption: {
    flex: 1,
    justifyContent: 'center',
  },
  listDesctiptionText: {
    textAlign: 'center',
  },
  userItem: {
    justifyContent: 'space-between',
    padding: USER_ITEM_VERTICAL_PADDING,
  },
  userItemContent: {
    flexShrink: 1,
  },
  overlayButtonWrapper: {
    position: Platform.OS === 'web' ? 'fixed' : 'absolute',
    bottom: (!isWebTouchDevice() && isWeb ? 20 : 0) + 8,
  },
  flexRow: {
    flexDirection: 'row',
    alignItems: 'center',
  },
})
